/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.ra;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.logging.Logger;

import javax.resource.ResourceException;
import javax.resource.spi.ActivationSpec;
import javax.resource.spi.BootstrapContext;
import javax.resource.spi.ResourceAdapter;
import javax.resource.spi.ResourceAdapterInternalException;
import javax.resource.spi.endpoint.MessageEndpointFactory;
import javax.transaction.xa.XAResource;

import org.drools.FactException;
import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.RuleIntegrationException;
import org.drools.WorkingMemory;
import org.drools.compiler.DroolsParserException;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderConfiguration;

public class DroolsResourceAdapter implements ResourceAdapter {

    private static Logger logger = Logger.getLogger(DroolsResourceAdapter.class
            .getName());

    public void start(BootstrapContext arg0)
            throws ResourceAdapterInternalException {

        DroolsWorkingMemory droolsWorkingMemory = DroolsWorkingMemory
                .getInstance();

        try {
            // Configuration of the packageBuilder
            logger
                    .info("Configuration of the packageBuiler for the new ruleBase...");
            PackageBuilderConfiguration pkgBuilderCfg = new PackageBuilderConfiguration();
            pkgBuilderCfg.setCompiler(PackageBuilderConfiguration.JANINO);

            RuleBase ruleBase = RuleBaseFactory.newRuleBase();

            PackageBuilder packageBuilder = new PackageBuilder(pkgBuilderCfg);

            logger.info("Load the file JASMINE_RULES/defaultRules.drl.xml");
            String location = System.getenv("JASMINE_RULES");
            if (location == null) {
                throw new Exception(
                        "The environment variable JASMINE_RULES does not exist");
            }
            File directory = new File(location);
            Boolean exist = false;
            if (directory.isDirectory()) {
                for (File file : directory.listFiles()) {
                    if (file.getName().equals("defaultRules.drl.xml")) {
                        exist = true;
                    }
                }
            }
            if (exist == false) {
                throw new Exception(
                        "The file defaultRules.drl.xml does not exist");
            }

            // Add the file defaultRules.drl.xml to the ruleBase
            packageBuilder.addPackageFromXml(new FileReader(location
                    + File.separator + "defaultRules.drl.xml"));

            ruleBase.addPackage(packageBuilder.getPackage());

            // Create and init a new WorkingMemory
            logger.info("Init the Working Memory...");
            WorkingMemory workingMemory = ruleBase.newWorkingMemory();
            droolsWorkingMemory.initWorkingMemory(workingMemory);

        } catch (RuleIntegrationException e) {
            e.printStackTrace();
        } catch (FactException e) {
            e.printStackTrace();
        } catch (DroolsParserException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void stop() {
        DroolsWorkingMemory droolsWorkingMemory = DroolsWorkingMemory
                .getInstance();
        droolsWorkingMemory.setStarted(false);
    }

    public void endpointActivation(MessageEndpointFactory arg0,
            ActivationSpec arg1) throws ResourceException {
    }

    public void endpointDeactivation(MessageEndpointFactory arg0,
            ActivationSpec arg1) {
    }

    public XAResource[] getXAResources(ActivationSpec[] arg0)
            throws ResourceException {
        return null;
    }
}
