/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

/**
 * 
 * @author Yann DAVIN <yann.davin@gmail.com>
 * 2006
 *
 */
package org.objectweb.jasmine.deployer;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * DeployerJade aims to provide external api to deploy, start and stop a
 * jadenode This deployer only work with a specific version of JADE(OSGi)
 * 
 * TODO generate execute.properties for Jadeboot and Jadenode on remote machine
 * At this time we suppose that etc/execute.properties file for jadeboot and
 * jadenode have been correctely configured (essentialy set -registryHost and
 * -ndHost with the ip or name of the host that will be used for launch
 * jadeboot)
 */

public class DeployerJadeImpl extends DeployerSSH implements DeployerJadeIfc {
    Logger logger = Logger.getLogger(this.getClass().getName());

    public DeployerJadeImpl(String user, String pass, String host) {
        super(user, pass, host);
    }

    private String jadeArchiveFile;

    private String destJadePath;

    private String jadeBootHost;

    public DeployerJadeImpl(String user, String pass, String host,
            String jadeArchiveFile, String destJadePath, String jadeBootHost) {
        super(user, pass, host);
        this.jadeArchiveFile = jadeArchiveFile;
        this.destJadePath = destJadePath;
        this.jadeBootHost = jadeBootHost;
    }

    public void copy() throws Exception {
        File archive = new File(jadeArchiveFile);

        if (!archive.exists()) {
            logger.severe("Jade archive file : " + archive.getAbsolutePath()
                    + " don't found.");
            return;
        }
        this.scp(archive.getAbsoluteFile().toString(), destJadePath + "/"
                + archive.getName());
        System.out.println("hihi");
        logger.log(Level.INFO, "Jade archive file copied.");
    }

    public void decompress() throws Exception {
        File archive = new File(jadeArchiveFile);
        this.exec("cd " + destJadePath + "; tar zxf " + archive.getName()
                + "; rm " + archive.getName());
    }

    public void install() throws Exception {
        this.copy();
        this.decompress();
    }

    public void uninstall() throws Exception {
        File archive = new File(jadeArchiveFile);
        this.stop();
        this.exec("cd " + destJadePath + "; rm -R Jade;");
    }

    public void configure() throws Exception {
        // TODO must generate jadenode execute.properties
        // because we need to know where is the jadeboot
    }

    public class ThreadJadeNode implements Runnable {
        public void run() {
            // We suppose that the arborescence of Jade is static.
            // But jade evolve so, we'll have to modify this unless
            // jade team develop their own api to deploy Jade
            exec("cd "
                    + destJadePath
                    + "/Jade/output/dist/Jade/examples/jadenode;ant start -DregistryHost="
                    + jadeBootHost + " -DregistryPort=1238" + " -ndHost="
                    + jadeBootHost + "-ndPort=9998" + ";");
        }
    };

    public class ThreadJadeBoot implements Runnable {
        public void run() {
            // We suppose that the arborescence of Jade is static.
            // But jade evolve so, we'll have to modify this unless
            // jade team develop their own api to deploy Jade
            exec("cd "
                    + destJadePath
                    + "/Jade/output/dist/Jade/examples/jadeboot;ant start -DregistryHost="
                    + jadeBootHost + " -DregistryPort=1238" + ";");
        }
    };

    public class ThreadJadeJmx implements Runnable {
        public void run() {
            // We suppose that the arborescence of Jade is static.
            // But jade evolve so, we'll have to modify this unless
            // jade team develop their own api to deploy Jade
            exec("cd " + destJadePath + "/Jade;ant jade-jmx;");
        }
    };

    Thread jadeNode;

    public void startJadeNode() throws Exception {
        jadeNode = new Thread(new ThreadJadeNode());
        jadeNode.start();
    }

    Thread jadeBoot;

    Thread jadeJmx;

    public void startJadeBoot() throws Exception {
        jadeBoot = new Thread(new ThreadJadeBoot());
        jadeBoot.start();

        // Launcher jmx agent to communicate with jadeboot via jmx
        jadeJmx = new Thread(new ThreadJadeJmx());
        jadeJmx.start();
    }

    public void stop() throws Exception {
        // we kill all jadenode (normaly we have only one jadenode for one host)
        // We the current version of JADE(OSGi) there is not way to stop a jade
        // node properly via the shell
        jadeNode = null;
        jadeBoot = null;
        jadeJmx = null;
        this.exec("kill -9 `jps | grep NodeLauncher | awk '{print $1}'`");
        this.exec("kill -9 `jps | grep AgentJade | awk '{print $1}'`");
    }

    // Example
    public static void main(String[] args) {
        DeployerJadeImpl deployer = new DeployerJadeImpl("testjasmine",
                "testjasmine", "frecb000707", "/download/tmp/Jade.tar.gz",
                "/home/testjasmine", "frecb000707");
        try {
            deployer.copy();
            deployer.decompress();
            deployer.startJadeBoot();
            // deployer.startJadeNode();
            // System.out.println("hihi");
            // Thread.sleep(1000*20);
            // deployer.stop();
            // deployer.uninstall();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
