/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.deployer;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.jasmine.deployer.DeployerJadeImpl.ThreadJadeBoot;
import org.objectweb.jasmine.deployer.DeployerJadeImpl.ThreadJadeJmx;
import org.objectweb.jasmine.deployer.DeployerJadeImpl.ThreadJadeNode;

public class DeployerProbesImpl extends DeployerSSH implements
        DeployerProbesIfc {
    Logger logger = Logger.getLogger(this.getClass().getName());

    /*
     * public DeployerProbesImpl(String user, String pass, String host){
     * super(user,pass,host); }
     */

    private String probesArchiveFile;

    private String destProbesPath;

    public DeployerProbesImpl(String user, String pass, String host,
            String probesArchiveFile, String destProbesPath) {
        super(user, pass, host);
        this.probesArchiveFile = probesArchiveFile;
        this.destProbesPath = destProbesPath;
    }

    public void copy() throws Exception {
        File archive = new File(probesArchiveFile);

        if (!archive.exists()) {
            logger.severe("archive file : " + archive.getAbsolutePath()
                    + " don't found.");
            return;
        }
        this.scp(archive.getAbsoluteFile().toString(), destProbesPath + "/"
                + archive.getName());
        logger.log(Level.INFO, "Archive file copied.");
    }

    public void decompress() throws Exception {
        File archive = new File(probesArchiveFile);
        this.exec("cd " + destProbesPath + "; tar zxf " + archive.getName()
                + "; rm " + archive.getName());
    }

    public void install() throws Exception {
        this.copy();
        this.decompress();
    }

    public void uninstall() throws Exception {
        File archive = new File(probesArchiveFile);
        this.stop();
        this.exec("cd " + destProbesPath + "; rm -R JasmineMonitoringSystem;");
    }

    public void configure() throws Exception {
    }

    public class ThreadProbes implements Runnable {
        String jmsUrl;

        String jmxPort;

        public ThreadProbes(String jmsUrl, String jmxPort) {
            this.jmsUrl = jmsUrl;
            this.jmxPort = jmxPort;
        }

        public void run() {
            // We suppose that the arborescence of Jade is static.
            // But jade evolve so, we'll have to modify this unless
            // jade team develop their own api to deploy Jade
            exec("export JASMINE_PROBES=" + destProbesPath
                    + "/JasmineMonitoringSystem;" + "cd " + destProbesPath
                    + "/JasmineMonitoringSystem;" + "ant configure -Djms-url="
                    + jmsUrl + " -Djmx-host=" + getHost() + " -Djmx-port="
                    + jmxPort + "; " + "ant  start ;");
        }
    };

    Thread probes;

    public void start(String jmsUrl, String jmxPort) throws Exception {
        probes = new Thread(new ThreadProbes(jmsUrl, jmxPort));
        probes.start();
    }

    public void stop() throws Exception {
        // we kill all jadenode (normaly we have only one jadenode for one host)
        // We the current version of JADE(OSGi) there is not way to stop a jade
        // node properly via the shell
        probes = null;
        this.exec("cd " + destProbesPath + "/JasmineMonitoringSystem;"
                + "ant stop;");
    }

    // Example
    public static void main(String[] args) {
        DeployerProbesIfc deployer = new DeployerProbesImpl("testjasmine",
                "testjasmine", "frecb000707",
                "/download/tmp/JasmineMonitoringSystem.tar.gz",
                "/home/testjasmine");
        try {
            // deployer.install();
            deployer.start("freci017089", "9099");
            // deployer.startJadeNode();
            // System.out.println("hihi");
            // Thread.sleep(1000*20);
            // deployer.stop();
            // deployer.uninstall();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
