/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

/**
 * @author Yann DAVIN <yann.davin@gmail.com>
 * 2006
 *
 */
package org.objectweb.jasmine.deployer;

import java.io.File;
import com.jcraft.jsch.*;
import javax.swing.*;
import java.io.*;

/**
 * deploy jade node and jade boot on wanted host
 */
public abstract class DeployerSSH {

    private String user = null;

    private String pass = null;

    private String host = null;

    public String getHost() {
        return host;
    }

    public DeployerSSH(String user, String pass, String host) {
        this.user = user;
        this.pass = pass;
        this.host = host;
    }

    /*
     * public abstract void install() throws Exception; public abstract void
     * startJadeNode() throws Exception; public abstract void startJadeBoot()
     * throws Exception; public abstract void stop() throws Exception; public
     * abstract void uninstall() throws Exception;
     */

    public class MyUserInfo implements UserInfo {
        public String getPassword() {
            return pass;
        }

        public boolean promptYesNo(String str) {
            return true;
        }

        String passwd;

        JTextField passwordField = (JTextField) new JPasswordField(20);

        public String getPassphrase() {
            return null;
        }

        public boolean promptPassphrase(String message) {
            return true;
        }

        public boolean promptPassword(String message) {
            return true;
        }

        public void showMessage(String message) {
        }
    }

    public void exec(String command) {
        try {
            JSch jsch = new JSch();

            Session session = jsch.getSession(user, host, 22);

            /*
             * String xhost="127.0.0.1"; int xport=0; String
             * display=JOptionPane.showInputDialog("Enter display name",
             * xhost+":"+xport); xhost=display.substring(0,
             * display.indexOf(':'));
             * xport=Integer.parseInt(display.substring(display.indexOf(':')+1));
             * session.setX11Host(xhost); session.setX11Port(xport+6000);
             */

            // username and password will be given via UserInfo interface.
            UserInfo ui = new MyUserInfo();
            session.setUserInfo(ui);
            session.connect();

            Channel channel = session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);
            channel.setXForwarding(true);

            // channel.setInputStream(System.in);
            channel.setInputStream(null);

            // channel.setOutputStream(System.out);

            // FileOutputStream fos=new FileOutputStream("/tmp/stderr");
            // ((ChannelExec)channel).setErrStream(fos);
            ((ChannelExec) channel).setErrStream(System.err);

            InputStream in = channel.getInputStream();

            channel.connect();

            byte[] tmp = new byte[1024];
            while (true) {
                while (in.available() > 0) {
                    int i = in.read(tmp, 0, 1024);
                    if (i < 0)
                        break;
                    System.out.print(new String(tmp, 0, i));
                }
                if (channel.isClosed()) {
                    System.out.println("exit-status: "
                            + channel.getExitStatus());
                    break;
                }
                try {
                    Thread.sleep(1000);
                } catch (Exception ee) {
                }
            }
            channel.disconnect();
            session.disconnect();
        } catch (Exception e) {
            System.out.println(e);
        }
    }

    public void scp(String src, String dest) throws Exception {
        FileInputStream fis = null;
        try {

            String lfile = src;
            String user = this.user;
            String host = this.host;
            String rfile = dest;

            JSch jsch = new JSch();
            Session session = jsch.getSession(user, host, 22);

            // username and password will be given via UserInfo interface.
            UserInfo ui = new MyUserInfo();
            session.setUserInfo(ui);
            session.connect();

            // exec 'scp -t rfile' remotely
            String command = "scp -p -t " + rfile;
            Channel channel = session.openChannel("exec");
            ((ChannelExec) channel).setCommand(command);

            // get I/O streams for remote scp
            OutputStream out = channel.getOutputStream();
            InputStream in = channel.getInputStream();

            channel.connect();

            if (checkAck(in) != 0) {
                throw new Exception("scp error : checkAck(in)!=0");
            }

            // send "C0644 filesize filename", where filename should not include
            // '/'
            long filesize = (new File(lfile)).length();
            command = "C0644 " + filesize + " ";
            if (lfile.lastIndexOf('/') > 0) {
                command += lfile.substring(lfile.lastIndexOf('/') + 1);
            } else {
                command += lfile;
            }
            command += "\n";
            out.write(command.getBytes());
            out.flush();
            if (checkAck(in) != 0) {
                System.exit(0);
            }

            // send a content of lfile
            fis = new FileInputStream(lfile);
            byte[] buf = new byte[1024];
            while (true) {
                int len = fis.read(buf, 0, buf.length);
                if (len <= 0)
                    break;
                out.write(buf, 0, len); // out.flush();
            }
            fis.close();
            fis = null;
            // send '\0'
            buf[0] = 0;
            out.write(buf, 0, 1);
            out.flush();
            if (checkAck(in) != 0) {
                System.exit(0);
            }
            out.close();

            channel.disconnect();
            session.disconnect();
        } catch (Exception e) {
            System.out.println(e);
            try {
                if (fis != null)
                    fis.close();
            } catch (Exception ee) {
                throw e;
            }
        }
    }

    static int checkAck(InputStream in) throws IOException {
        int b = in.read();
        // b may be 0 for success,
        // 1 for error,
        // 2 for fatal error,
        // -1
        if (b == 0)
            return b;
        if (b == -1)
            return b;

        if (b == 1 || b == 2) {
            StringBuffer sb = new StringBuffer();
            int c;
            do {
                c = in.read();
                sb.append((char) c);
            } while (c != '\n');
            if (b == 1) { // error
                System.out.print(sb.toString());
            }
            if (b == 2) { // fatal error
                System.out.print(sb.toString());
            }
        }
        return b;
    }

    public static void main(String[] args) {
        /*
         * DeployerSSHImpl deployer = new
         * DeployerSSHImpl("testjasmine","testjasmine","frecb000707"); try{
         * deployer.scp("/home/daviny/test", "/home/testjasmine/test");
         * deployer.exec("ls"); }catch(Exception e){e.printStackTrace();}
         */
    }
}
