/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.jade.ejb;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.Context;

import com.sun.jndi.rmi.registry.RegistryContextFactory;

/**
*
* @author Yann Davin (yann.davin@gmail.com)
*
* 2006
*/
@Stateless
@Remote(JadeEJBRemote.class)
public class JadeEJB implements JadeEJBRemote {

    /**
     *
     */
    private static String jadeBeanName = "Jade:name=manager";

    /**
     *
     */
    private static ObjectName jadeBean = null;

    /**
     *
     */
    private static MBeanServerConnection connection = null;

    // ------------------------------------------------------------------------
    // Unidentified Method :-) (to clean)
    // ------------------------------------------------------------------------

    /**
     * @param url
     */
    public final void setJMXURL(final String url) {
        this.setJadeBean(url);
    }

    // ------------------------------------------------------------------------
    // Implementation of the JadeEJBRemote interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#setJMXURL(java.lang.String,
     *      java.lang.String)
     */
    public final void setJMXURL(final String host, final String port) {
        this.setJadeBean("service:jmx:rmi:///jndi/rmi://" + host + ":" + port
                + "/server");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#deploy(java.lang.String,
     *      java.util.Map)
     */
    public final int deploy(final String idAdl,
            final Map<String, String> context) throws Exception {
        try {
            return (Integer) connection.invoke(jadeBean, "deploy",
                    new Object[] {idAdl, context}, new String[] {
                            String.class.getName(), Map.class.getName()});

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#deploy(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public final int deploy(final String idAdl, final String host,
            final String jadeNodeNumber) throws Exception {
        try {
            return (Integer) connection.invoke(jadeBean, "deploy",
                    new Object[] {idAdl, host, jadeNodeNumber}, new String[] {
                            String.class.getName(), String.class.getName(),
                            String.class.getName()});

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#deploy(java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public final int deploy(final String idAdl, final String host,
            final String dirLocal, final String jadeNodeNumber)
            throws Exception {
        try {
            return (Integer) connection.invoke(jadeBean, "deploy",
                    new Object[] {idAdl, host, dirLocal, jadeNodeNumber},
                    new String[] {String.class.getName(),
                            String.class.getName(), String.class.getName(),
                            String.class.getName()});

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#start(org.objectweb.fractal.api.Component)
     */
    public final void start(final int componentID) {
        try {

            connection.invoke(jadeBean, "start", new Object[] {componentID},
                    new String[] {Integer.TYPE.getName()});
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#stop(org.objectweb.fractal.api.Component)
     */
    public final void stop(final int componentID) {
        try {

            connection.invoke(jadeBean, "stop", new Object[] {componentID},
                    new String[] {Integer.TYPE.getName()});

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#getAttribute(java.lang.String,
     *      java.lang.String)
     */
    public final String getAttribute(final int componentID,
            final String attribute) {
        try {
            return (String) connection.invoke(jadeBean, "getAttribute",
                    new Object[] {componentID, attribute}, new String[] {
                            Integer.TYPE.getName(), String.class.getName()});
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#setAttribute(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public final void setAttribute(final int componentID,
            final String attribute, final String value) {
        try {
            connection.invoke(jadeBean, "setAttribute", new Object[] {
                    componentID, attribute, value}, new String[] {
                    Integer.TYPE.getName(), String.class.getName(),
                    String.class.getName()});
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#listAttributes(java.lang.String)
     */
    public final String[] listAttributes(final String name) {
        try {
            return (String[]) connection.invoke(jadeBean, "listAttributes",
                    new Object[] {name}, new String[] {String.class.getName()});
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#setRegistry(java.lang.String,
     *      int)
     */
    public final void setRegistry(final String host, final int port) {
        try {
            connection.invoke(jadeBean, "setRegistry",
                    new Object[] {host, port}, new String[] {
                            String.class.getName(), Integer.TYPE.getName()});
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#bind(java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public final void bind(final String component1, final String itf1,
            final String component2, final String itf2) throws Exception {
        connection.invoke(jadeBean, "unbind", new Object[] {component1, itf1,
                component2, itf2}, new String[] {String.class.getName(),
                String.class.getName()});
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#unbind(java.lang.String,
     *      java.lang.String)
     */
    public final void unbind(final String component1, final String itf1)
            throws Exception {
        connection.invoke(jadeBean, "unbind", new Object[] {component1, itf1},
                new String[] {String.class.getName(), String.class.getName()});
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.ejb.JadeEJBRemote#listFc(java.lang.String)
     */
    public final String[] listFc(final String component1) throws Exception {
        return (String[]) connection.invoke(jadeBean, "listFc",
                new Object[] {component1},
                new String[] {String.class.getName()});
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * @param url
     */
    private void setJadeBean(final String url) {
        connection = getConnection(url);
        try {
            jadeBean = ObjectName.getInstance(jadeBeanName);
        } catch (MalformedObjectNameException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * @param url
     * @return
     */
    private static MBeanServerConnection getConnection(final String url) {
        JMXServiceURL jmxUrl = null;
        try {
            jmxUrl = new JMXServiceURL(url);
        } catch (MalformedURLException e) {
            throw new RuntimeException(e);
        }

        // System.setProperty(Context.INITIAL_CONTEXT_FACTORY,
        // RegistryContextFactory.class.getName());

        HashMap<String, String> env = new HashMap<String, String>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, RegistryContextFactory.class
                .getName());

        try {
            return JMXConnectorFactory.connect(jmxUrl, env)
                    .getMBeanServerConnection();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
}
