/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.jade.jmx;

import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;

/**
*
* @author Yann Davin (yann.davin@gmail.com)
*
* 2006
*/
public class AgentJade {

    /**
     *
     */
    private MBeanServer mbs = null;
    /**
     *
     */
    private ObjectName managerName = null;
    /**
     *
     */
    private JMXConnectorServer cs = null;
    /**
     *
     */
    private Registry registry = null;
    /**
     *
     */
    private String jmxServiceUrl;

    // ------------------------------------------------------------------------
    // Public methods
    // ------------------------------------------------------------------------

    /**
     * @param url the JMX service URL
     */
    public final void setJmxServiceUrl(final String url) {
        this.jmxServiceUrl = url;
    }

    /**
     * @return the JMX service URL
     */
    public final String getJmxServiceUrl() {
        return this.jmxServiceUrl;
    }

    /**
     * Start the JMX agent connected to Jade.
     */
    public final void start() {
        try {
            String[] splits = this.jmxServiceUrl.split("/");
            String[] address = splits[6].split(":");
            // String host = address[0];
            String port = address[1];
            registry = java.rmi.registry.LocateRegistry.createRegistry(Integer
                    .parseInt(port));
            System.out.println("RMI registry ready.\n");
        } catch (Exception e) {
            e.printStackTrace();
        }

        mbs = ManagementFactory.getPlatformMBeanServer();
        // HtmlAdaptorServer adapter = new HtmlAdaptorServer();

        // Unique identification of MBeans
        JadeJMXMBean managerBean = new JadeJMX();

        try {
            // Uniquely identify the MBeans and register them with the
            // MBeanServer
            managerName = new ObjectName("Jade:name=manager");

            mbs.registerMBean(managerBean, managerName);
            // Register and start the HTML adaptor
            // Create an RMI connector and start it
            JMXServiceURL url = new JMXServiceURL(jmxServiceUrl);
            cs = JMXConnectorServerFactory
                    .newJMXConnectorServer(url, null, mbs);
            cs.start();

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Stop the JMX agent connected to Jade.
     */
    public final void stop() {
        try {
            cs.stop();
            mbs.unregisterMBean(managerName);
            UnicastRemoteObject.unexportObject(registry, true);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // ------------------------------------------------------------------------
    // Overriden methods
    // ------------------------------------------------------------------------

    /* (non-Javadoc)
     * @see java.lang.Object#finalize()
     */
    @Override
    protected final void finalize() throws Throwable {
        // TODO close the connections
        /*
         * try{ cs.stop(); mbs.unregisterMBean(managerName);
         * UnicastRemoteObject.unexportObject(registry,true); }catch(Exception
         * e){e.printStackTrace();}
         */
        super.finalize();
    }

    // ------------------------------------------------------------------------
    // Main method
    // ------------------------------------------------------------------------

    /**
     * Launch MBean for JADE JMX management.
     *
     * @param args
     */
    public static void main(final String[] args) {
        AgentJade jade = new AgentJade();
        // TODO remove port and host from the code
        try {
            String url = "service:jmx:rmi:///jndi/rmi://"
                    + InetAddress.getLocalHost().getHostAddress() + ":"
                    + System.getProperty("jade.rmi.port") + "/server";
            System.out.println(url);
            jade.setJmxServiceUrl(url);
            jade.start();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
