/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.jade.jmx;

import java.net.InetAddress;
import java.util.HashMap;
import java.util.Map;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.NameController;
import org.objectweb.fractal.rmi.registry.NamingService;
import org.objectweb.fractal.rmi.registry.Registry;
import org.objectweb.fractal.util.Fractal;
import org.objectweb.jasmine.jade.service.deployer.DeployerService;
import org.ow2.jasmine.jade.fractal.api.control.GenericAttributeController;

// TODO Use SpoonJMX to generate MBean with annotation, use full permit
// to add some info about param and method that will be displayed with jconsole
/**
 *
 * @author Yann Davin (yann.davin@gmail.com)
 *
 * 2006
 */
public class JadeJMX implements JadeJMXMBean {

    /**
     *
     */
    private static NamingService namingService = null;

    /**
     * Map containing deployed components references and their ID.
     */
    private Map<Integer, Component> deployedComponents = new HashMap<Integer, Component>();;

    // ------------------------------------------------------------------------
    // Implementation of the JadeIfc interface
    // ------------------------------------------------------------------------

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.JadeIfc#deploy(java.lang.String,
     *      java.util.HashMap)
     */
    public int deploy(String idAdl, HashMap context) throws Exception {

        try {
            Component c = getDeployer().deploy(idAdl, context);

            deployedComponents.put(c.hashCode(), c);

            return c.hashCode();

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.JadeIfc#deploy(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public int deploy(String idAdl, String host, String jadeNodeNumber)
            throws Exception {

        HashMap<String, String> cxt = new HashMap<String, String>();
        cxt.put("host", InetAddress.getAllByName(host)[0]
                .getCanonicalHostName());
        cxt.put("number", jadeNodeNumber);

        try {

            int res = deploy(idAdl, cxt);
            System.out.println("deploy(" + idAdl + ", " + host + ", "
                    + jadeNodeNumber + ") = " + res);
            return res;

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.JadeIfc#deploy(java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public int deploy(String idAdl, String host, String dirLocal,
            String jadeNodeNumber) throws Exception {

        HashMap<String, String> cxt = new HashMap<String, String>();
        cxt.put("host", InetAddress.getAllByName(host)[0]
                .getCanonicalHostName());
        cxt.put("number", jadeNodeNumber);
        cxt.put("dirLocal", dirLocal);

        try {

            int res = deploy(idAdl, cxt);
            System.out.println("deploy(" + idAdl + ", " + host + ", "
                    + jadeNodeNumber + ", " + dirLocal + ") = " + res);
            return res;

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#start(int)
     */
    public void start(int componentID) throws Exception {

        try {
            Component c = deployedComponents.get(componentID);
            getDeployer().start(c);

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        System.out.println(componentID + " : start");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#stop(int)
     */
    public void stop(int componentID) throws Exception {

        try {
            Component c = deployedComponents.get(componentID);
            getDeployer().stop(c);

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        System.out.println(componentID + " : stop");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#getAttribute(java.lang.String,
     *      java.lang.String)
     */
    public String getAttribute(int componentID, String attribute)
            throws Exception {

        try {
            Component comp = deployedComponents.get(componentID);
            GenericAttributeController resource = (GenericAttributeController) comp
                    .getFcInterface("attributes");
            // GenericAttributeController resource = FractalUtil
            // .getGenericAttributeController(comp);
            String res = resource.getAttribute(attribute);
            System.out.println(componentID + " : getAttribute(" + attribute
                    + ")");
            return res;
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#getDeployer()
     */
    public DeployerService getDeployer() throws Exception {
        try {
            Component jadeboot = getJadeBoot();
            if (jadeboot == null) {
                return null;
            }

            Component managedResource = getSubComponent(jadeboot,
                    "managed_resources");
            Component resourceDeployer = getSubComponent(managedResource,
                    "resource_deployer");
            return (DeployerService) resourceDeployer
                    .getFcInterface("deployer_service");
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#getJadeBoot()
     */
    public Component getJadeBoot() throws Exception {

        Component jadeboot = null;
        if (namingService != null) {
            try {
                jadeboot = namingService.lookup("jadeboot");
            } catch (Exception e) {
                e.printStackTrace();
                throw e;
            }
        }
        return jadeboot;

    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#listAttributes(java.lang.String)
     */
    public String[] listAttributes(String name) throws Exception {

        try {
            Component comp = getComponent(name);
            GenericAttributeController resource = (GenericAttributeController) comp
                    .getFcInterface("attributes");
            return resource.listFcAtt();
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#setAttribute(java.lang.String,
     *      java.lang.String, java.lang.String)
     */
    public void setAttribute(int componentID, String attribute, String value)
            throws Exception {

        try {
            Component comp = deployedComponents.get(componentID);
            GenericAttributeController resource = (GenericAttributeController) comp
                    .getFcInterface("attributes");

            // GenericAttributeController resource = FractalUtil
            // .getGenericAttributeController(comp);

            resource.setAttribute(attribute, value);

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        System.out.println(componentID + " : setAttribute(" + attribute + ", "
                + value + ")");
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#setRegistry(java.lang.String,
     *      int)
     */
    public void setRegistry(String host, int port) throws Exception {
        try {
            namingService = Registry.getRegistry(host, port);
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.JadeIfc#bind(java.lang.String,
     *      java.lang.String, java.lang.String, java.lang.String)
     */
    public void bind(String component1, String itf1, String component2,
            String itf2) throws Exception {

        try {
            Component c1 = getComponent(component1);
            Component c2 = getComponent(component2);
            NameController n1 = (NameController) c1
                    .getFcInterface("name-controller");
            NameController n2 = (NameController) c1
                    .getFcInterface("name-controller");

            BindingController bc = (BindingController) c1
                    .getFcInterface("binding-controller");
            BindingController bc2 = (BindingController) c2
                    .getFcInterface("binding-controller");

            try {
                bc.bindFc(itf1, bc2.lookupFc(itf2));
            } catch (Exception ee) {
                try {
                    bc.bindFc(itf1, c2.getFcInterface(itf2));
                } catch (Exception eee) {
                    throw eee;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#unbind(java.lang.String,
     *      java.lang.String)
     */
    public void unbind(String component1, String itf1) throws Exception {

        Component c1 = getComponent(component1);
        BindingController bc = Fractal.getBindingController(c1);
        bc.unbindFc(itf1);
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.jasmine.jade.jmx.JadeJMXMBean#listFc(java.lang.String)
     */
    public String[] listFc(String component1) throws Exception {

        try {
            Component c1 = getComponent(component1);

            BindingController bc = (BindingController) c1
                    .getFcInterface("binding-controller");
            return bc.listFc();
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
    }

    // ------------------------------------------------------------------------
    // Private methods
    // ------------------------------------------------------------------------

    /**
     * @param path
     * @return
     * @throws Exception
     */
    private static Component getComponent(String path) throws Exception {
        String[] splits = path.split("/");
        Component result = null;
        try {
            Component d = result = namingService.lookup(splits[0]);
            for (int i = 1; i < splits.length; i++) {
                result = getSubComponent(d, splits[i]);
                d = result;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * @param c
     * @param instanceComponentName
     * @return
     */
    private static Component getSubComponent(Component c,
            String instanceComponentName) {
        if (c == null) {
            return null;
        }
        if (instanceComponentName == null) {
            return null;
        }
        try {
            Component[] components = Fractal.getContentController(c)
                    .getFcSubComponents();
            for (int i = 0; i < components.length; i++) {
                if (instanceComponentName.equals(Fractal.getNameController(
                        components[i]).getFcName())) {
                    return components[i];
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        return null;
    }

}
