/**
 * JASMINe
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: jasmine@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:$
 * --------------------------------------------------------------------------
 */

package org.objectweb.jasmine.rules.probes;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.TextMessage;
import javax.xml.xpath.XPathConstants;

import org.drools.FactException;
import org.drools.WorkingMemory;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.objectweb.jasmine.ra.DroolsWorkingMemory;
import org.objectweb.jasmine.rules.probes.helpers.CpuProbe;
import org.objectweb.jasmine.rules.probes.helpers.MemoryProbe;
import org.objectweb.jasmine.rules.probes.helpers.NetworkProbe;
import org.objectweb.jasmine.rules.probes.parsers.CpuParser;
import org.objectweb.jasmine.rules.probes.parsers.MemoryParser;
import org.objectweb.jasmine.rules.probes.parsers.NetworkParser;
import org.objectweb.jasmine.rules.probes.parsers.ParserProbe;

@MessageDriven(activationConfig = {
        @ActivationConfigProperty(propertyName = "destination", propertyValue = "sampleTopic"),
        @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Topic") })
public class MessageDrivenBean implements MessageListener {

    private ParserProbe parserProbe;

    /**
     * Passes a message to the listener.
     * 
     * @param message -
     *            the message passed to the listener
     */
    public void onMessage(final Message message) {

        if (message instanceof TextMessage) {
            try {

                DroolsWorkingMemory droolsWorkingMemory = DroolsWorkingMemory
                        .getInstance();
                WorkingMemory workingMemory = droolsWorkingMemory
                        .getWorkingMemory();

                parserProbe = new ParserProbe();
                String xml = "<root>" + ((TextMessage) message).getText()
                        + "</root>";

                NodeList nodes = (NodeList) parserProbe.evaluateSAX(xml,
                        "/root/probe", XPathConstants.NODESET);
                if (nodes != null) {
                    for (int i = 0; i < nodes.getLength(); ++i) {
                        Node node = nodes.item(i);

                        NodeList enfants = node.getChildNodes();
                        for (int k = 0; k < enfants.getLength(); ++k) {
                            Node enfant = enfants.item(k);

                            if (enfant.getNodeName().equals("name")) {

                                if (enfant.getTextContent().equals("cpu")) {
                                    CpuParser cpuParser = new CpuParser();
                                    CpuProbe cpuProbe = cpuParser
                                            .createCpuProbe(node);
                                    workingMemory.assertObject(cpuProbe);

                                }

                                if (enfant.getTextContent().equals("memory")) {
                                    MemoryParser memoryParser = new MemoryParser();
                                    MemoryProbe memoryProbe = memoryParser
                                            .createMemoryProbe(node);
                                    workingMemory.assertObject(memoryProbe);
                                }

                                if (enfant.getTextContent().equals("network")) {
                                    NetworkParser networkParser = new NetworkParser();
                                    NetworkProbe networkProbe = networkParser
                                            .createNetworkProbe(node);

                                    workingMemory.assertObject(networkProbe);
                                }
                            }
                        }
                    }
                }

            } catch (JMSException e) {
                System.err
                        .println("Error while getting the content of the message");
                e.printStackTrace();
            } catch (FactException e) {
                e.printStackTrace();
            }
        }

    }

}
